@@ -34,6 +34,38 @@ use std::{
 /// function](https://en.wikipedia.org/wiki/Inverse_Ackermann_function). For all practical
 /// purposes, you can think of each operation as being essentially `O(1)`.
 ///
+/// # Accessing the Disjoint Sets
+///
+/// In order to allow easy access to the disjoint sets stored in the forest, this struct implements
+/// `IntoIterator`:
+/// ```
+/// use union_find_rs::prelude::*;
+/// use std::collections::HashSet;
+///
+/// let mut sets: DisjointSets<usize> = DisjointSets::new();
+///
+/// sets.make_set(1).unwrap();
+/// sets.make_set(4).unwrap();
+/// sets.make_set(9).unwrap();
+///
+/// sets.union(4, 9).unwrap();
+///
+/// // `into_iter` returns an iterator whose item is of type `HashSet<T>`
+/// let as_vec: Vec<HashSet<usize>> = sets.into_iter().collect();
+///
+/// // there should be 2 disjoint sets, where one of them only contains `4` and the other one
+/// // only contains `9`
+/// assert_eq!(as_vec.len(), 2);
+/// assert!(as_vec
+///     .iter()
+///     .find(|set| set == &&vec![1].into_iter().collect::<HashSet<usize>>())
+///     .is_some());
+/// assert!(as_vec
+///     .iter()
+///     .find(|set| set == &&vec![4, 9].into_iter().collect::<HashSet<usize>>())
+///     .is_some());
+/// ```
+///
 /// # References
 ///
 /// This struct was implemented using ideas from the following:
@@ -56,9 +88,9 @@ where
     fn make_set(&mut self, item: T) -> Result<()> {
         //
         match self.contains(item) {
-            //
+            // if `item` alreday exists, error with the approriate variant
             true => Err(Error::ItemAlreadyExists),
-            //
+            // only make the set if it doesn't already exist
             false => {
                 //
                 let node = Node::new(item, 0);
@@ -73,9 +105,9 @@ where
     fn union(&self, x: T, y: T) -> Result<()> {
         //
         match x == y {
-            //
+            // if they are equivalent, no need to merge their sets
             true => Ok(()),
-            //
+            // merge the two sets by first finding their representatives
             false => self.link(
                 self.find_set(x)?, //
                 self.find_set(y)?, //
@@ -87,22 +119,25 @@ where
     fn find_set(&self, item: T) -> Result<T> {
         //
         match self.get_node(&item) {
-            //
+            // `item` exists in this `DisjointSets` and is stored in `node`
             Some(node) => {
                 //
                 debug_assert!(item == node.get_item());
 
                 //
                 match node.get_parent() {
-                    // `node` is not a root node, so
-                    // 1. do path compression
-                    // 2. set parent to the root node
+                    // `node` is not a root node, so find the representative nad do path
+                    // compression
                     Some(parent) => {
-                        let root = self.find_set(parent)?;
-                        node.set_parent(root);
-                        Ok(root)
+                        // 1. find the representative by recursing
+                        let representative = self.find_set(parent)?;
+                        // 2. set the parent of `node` to the representative
+                        node.set_parent(representative);
+                        //
+                        Ok(representative)
                     }
-                    // `node` is a root node, so return its item directly
+                    // `node` is a root node, so return `item` since it's the representative of its
+                    // set
                     None => Ok(item),
                 }
             }
@@ -263,32 +298,36 @@ where
         self.nodes.capacity()
     }
 
-    //
+    // return the `Node` representation of `item`, if it exists
     fn get_node(&self, item: &T) -> Option<&Node<T>> {
         self.nodes.get(item)
     }
 
-    // link the two sets for which `x` and `y` are representatives
+    // merge the two sets for which `x` and `y` are representatives
     fn link(&self, x: T, y: T) -> Result<()> {
         // assert that `x` and `y` are indeed root nodes
         debug_assert!(self.get_node(&x).unwrap().get_parent().is_none());
         debug_assert!(self.get_node(&y).unwrap().get_parent().is_none());
+        // `x` should != `y` because of the check in `DisjointSets::union`
+        debug_assert!(x != y);
         //
         match self.nodes.contains_key(&x) && self.nodes.contains_key(&y) {
-            //
+            // both items exist, so link them
             true => {
-                //
+                // node representations of the items
                 let x_node = self.get_node(&x).unwrap();
                 let y_node = self.get_node(&y).unwrap();
-                //
+                // ranks of both of the nodes
                 let x_rank = x_node.get_rank();
                 let y_rank = y_node.get_rank();
-                //
+
+                // union-by-rank
                 match x_rank.cmp(&y_rank) {
                     // node with the larger rank becomes the parent
                     Greater => y_node.set_parent(x),
                     Less => x_node.set_parent(y),
-                    // choose the parent arbitrarily, and increment the parent's rank
+                    // if both ranks are equal, then choose the parent arbitrarily
+                    // and increment the parent's rank by 1
                     Equal => {
                         x_node.set_parent(y);
                         y_node.set_rank(y_rank + 1);
@@ -297,13 +336,41 @@ where
                 //
                 Ok(())
             }
-            //
+            // one or more of the items does not exist, so error
             false => Err(Error::ItemDoesNotExist),
         }
     }
 }
 
-// and we'll implement IntoIterator
+/// In order to allow easy access to the disjoint sets stored in the forest, this struct implements
+/// `IntoIterator`:
+/// ```
+/// use union_find_rs::prelude::*;
+/// use std::collections::HashSet;
+///
+/// let mut sets: DisjointSets<usize> = DisjointSets::new();
+///
+/// sets.make_set(1).unwrap();
+/// sets.make_set(4).unwrap();
+/// sets.make_set(9).unwrap();
+///
+/// sets.union(4, 9).unwrap();
+///
+/// // `into_iter` returns an iterator whose item is of type `HashSet<T>`
+/// let as_vec: Vec<HashSet<usize>> = sets.into_iter().collect();
+///
+/// // there should be 2 disjoint sets, where one of them only contains `4` and the other one
+/// // only contains `9`
+/// assert_eq!(as_vec.len(), 2);
+/// assert!(as_vec
+///     .iter()
+///     .find(|set| set == &&vec![1].into_iter().collect::<HashSet<usize>>())
+///     .is_some());
+/// assert!(as_vec
+///     .iter()
+///     .find(|set| set == &&vec![4, 9].into_iter().collect::<HashSet<usize>>())
+///     .is_some());
+/// ```
 impl<T> IntoIterator for DisjointSets<T>
 where
     T: Copy + Eq + Hash,
